use std::ffi::CString;
use ash::{vk, Entry};

pub fn use_layers(
    entry: &Entry,
    layers_to_select: Vec<&str>
) -> Vec<*const std::ffi::c_char> {
    let layers_available = unsafe {
        entry
            .enumerate_instance_layer_properties()
            .unwrap_or_default()
    };

    let mut selected_layers = Vec::<String>::new();

    log::info!("Available layers ({}):", layers_available.len());
    for l in layers_available {
        let layer_name = l
            .layer_name_as_c_str()
            .unwrap_or_default()
            .to_string_lossy()
            .into_owned();

        if layers_to_select.iter().any(|ln| (*ln).eq(&layer_name)) {
            selected_layers.push(layer_name);
        }

        log::info!(
            "\t{:?}\tImplementation version: {}\tVulkan Version: {}\tDescription: {:?}",
            l.layer_name_as_c_str().unwrap_or_default(),
            l.implementation_version,
            print_version(l.spec_version),
            l.description_as_c_str().unwrap_or_default()
        );
    }
    log::info!(""); // Add blank line

    log::info!("Layers wanted ({}):", layers_to_select.len());
    for l in layers_to_select {
        let selected = selected_layers.iter().any(|sl| sl.eq(l));
        log::info!("\t{:?}\tSelected: {}", l, selected);
    }
    log::info!(""); // Add blank line
    
    selected_layers.iter().map(|sl| {
        let c_string = CString::new(sl.clone().as_bytes()).unwrap();
        c_string.as_ptr()
    }).collect()
}

pub fn print_version(version: u32) -> String {
    format!(
        "{}.{}.{}",
        vk::api_version_major(version),
        vk::api_version_minor(version),
        vk::api_version_patch(version)
    )
}
